@@ -1,5 +1,5 @@
 /*
-** ypserver.c			YP server routines.
+** server.c			YP server routines.
 **
 ** Copyright (c) 1993 Signum Support AB, Sweden
 **
@@ -38,10 +38,6 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 
-#ifndef _PATH_YP
-#define _PATH_YP   "/var/yp"
-#endif
-char *path_ypdb = _PATH_YP;
 
 int debug_flag = 0;
 int dns_flag   = 0;
@@ -114,18 +110,17 @@ void *ypproc_null_2_svc(void *dummy,
 */
 static int is_valid_domain(const char *domain)
 {
-    char buf[1025];
     struct stat sbuf;
     
 
-    if (strcmp(domain, "binding") == 0 ||
+    if (domain == NULL ||
+	strcmp(domain, "binding") == 0 ||
 	strcmp(domain, "..") == 0 ||
 	strcmp(domain, ".") == 0 ||
 	strchr(domain, '/'))
 	return 0;
     
-    sprintf(buf, "%s/%s", path_ypdb, domain);
-    if (stat(buf, &sbuf) < 0 || !S_ISDIR(sbuf.st_mode))
+    if (stat(domain, &sbuf) < 0 || !S_ISDIR(sbuf.st_mode))
 	return 0;
 
     return 1;
@@ -141,8 +136,6 @@ bool_t *ypproc_domain_2_svc(domainname *name,
     
     
     rqhost = svc_getcaller(rqstp->rq_xprt);
-    if (!is_valid_host(rqhost))
-	return NULL;
 	
     if (debug_flag)
 	fprintf(stderr, "ypproc_domain(\"%s\") [From: %s:%d]\n",
@@ -150,6 +143,14 @@ bool_t *ypproc_domain_2_svc(domainname *name,
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
 
+    if (!is_valid_host(rqhost))
+    {
+	if (debug_flag)
+	    fprintf(stderr, "\t-> Ignored (not a valid source host)\n");
+	
+	return NULL;
+    }
+    
     if (is_valid_domain(*name))
 	result = TRUE;
     else
@@ -171,19 +172,25 @@ bool_t *ypproc_domain_nonack_2_svc(domainname *name,
 
     
     rqhost = svc_getcaller(rqstp->rq_xprt);
-    if (!is_valid_host(rqhost))
-	return NULL;
 
     if (debug_flag)
 	fprintf(stderr, "ypproc_domain_nonack(\"%s\") [From: %s:%d]\n",
 		*name,
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
-    
+
+    if (!is_valid_host(rqhost))
+    {
+	if (debug_flag)
+	    fprintf(stderr, "\t-> Ignored (not a valid source host)\n");
+	
+	return NULL;
+    }
+
     if (!is_valid_domain(*name))
     {
 	if (debug_flag)
-	    fprintf(stderr, "\t-> Not a valid domain.\n");
+	    fprintf(stderr, "\t-> Ignored (not a valid domain)\n");
 
 	/* Bail out and don't return any RPC value */
 	return NULL;
@@ -207,17 +214,12 @@ static GDBM_FILE open_database(const char *domain,
     char buf[1025];
 
 
-    /* Better be safe than sorry... */
-    if (strcmp(domain, "binding") == 0 ||
-	strcmp(domain, "..") == 0 ||
-	strcmp(domain, ".") == 0 ||
-	strchr(domain, '/'))
-	return 0;
-
     if (map[0] == '.' || strchr(map, '/'))
 	return 0;
 
-    sprintf(buf, "%s/%s/%s", path_ypdb, domain, map);
+    strcpy(buf, domain);
+    strcat(buf, "/");
+    strcat(buf, map);
     
     dbp = gdbm_open(buf, 0, O_RDONLY, 0, NULL);
 
@@ -259,7 +261,7 @@ int read_database(GDBM_FILE dbp,
     
     if (ckey.dptr == NULL)
     {
-	return YP_NOKEY;
+	return (flags & F_NEXT) ? YP_NOMORE : YP_NOKEY;
     }
 
     while (1)
@@ -318,7 +320,11 @@ static unsigned long get_dtm(const char *domain,
     struct stat sbuf;
     char buf[1025];
 
-    sprintf(buf, "%s/%s/%s", path_ypdb, domain, map);
+    
+    strcpy(buf, domain);
+    strcat(buf, "/");
+    strcat(buf, map);
+    
     if (stat(buf, &sbuf) < 0)
 	return 0;
     else
@@ -337,8 +343,6 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key,
 
     
     rqhost = svc_getcaller(rqstp->rq_xprt);
-    if (!is_valid_host(rqhost))
-	return NULL;
 
     if (debug_flag)
     {
@@ -355,6 +359,14 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key,
 		key->key.keydat_val);
     }
 
+    if (!is_valid_host(rqhost))
+    {
+	if (debug_flag)
+	    fprintf(stderr, "\t-> Ignored (not a valid source host)\n");
+	
+	return NULL;
+    }
+
     result.val.valdat_len = 0;
     if (result.val.valdat_val)
     {
@@ -483,8 +495,6 @@ ypresp_key_val *ypproc_first_2_svc(ypreq_nokey *key,
     
 
     rqhost = svc_getcaller(rqstp->rq_xprt);
-    if (!is_valid_host(rqhost))
-	return NULL;
 
     if (debug_flag)
     {
@@ -501,6 +511,14 @@ ypresp_key_val *ypproc_first_2_svc(ypreq_nokey *key,
 #endif
     }
 
+    if (!is_valid_host(rqhost))
+    {
+	if (debug_flag)
+	    fprintf(stderr, "\t-> Ignored (not a valid source host)\n");
+	
+	return NULL;
+    }
+
     result.key.keydat_len = 0;
     if (result.key.keydat_val)
     {
@@ -568,12 +586,10 @@ ypresp_key_val *ypproc_next_2_svc(ypreq_key *key,
     
     
     rqhost = svc_getcaller(rqstp->rq_xprt);
-    if (!is_valid_host(rqhost))
-	return NULL;
 
     if (debug_flag)
     {
-	fprintf(stderr, "ypproc_match(): [From: %s:%d]\n",
+	fprintf(stderr, "ypproc_next(): [From: %s:%d]\n",
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
 		
@@ -584,6 +600,14 @@ ypresp_key_val *ypproc_next_2_svc(ypreq_key *key,
 		key->key.keydat_val);
     }
 
+    if (!is_valid_host(rqhost))
+    {
+	if (debug_flag)
+	    fprintf(stderr, "\t-> Ignored (not a valid source host)\n");
+	
+	return NULL;
+    }
+
     result.key.keydat_len = 0;
     if (result.key.keydat_val)
     {
@@ -614,8 +638,8 @@ ypresp_key_val *ypproc_next_2_svc(ypreq_key *key,
 	{
 	    dkey.dsize = key->key.keydat_len;
 	    dkey.dptr  = key->key.keydat_val;
-	    
-	    result.stat = read_database(dbp, &dkey, &okey, &dval, 0);
+
+	    result.stat = read_database(dbp, &dkey, &okey, &dval, F_NEXT);
 
 	    if (result.stat == YP_TRUE)
 	    {
@@ -666,8 +690,6 @@ ypresp_xfr *ypproc_xfr_2_svc(ypreq_xfr *xfr,
 
 
     rqhost = svc_getcaller(rqstp->rq_xprt);
-    if (!is_valid_host(rqhost))
-	return NULL;
 
     if (debug_flag)
     {
@@ -682,6 +704,14 @@ ypresp_xfr *ypproc_xfr_2_svc(ypreq_xfr *xfr,
 	fprintf(stderr, "\t-> Request denied.\n");
     }
 
+    if (!is_valid_host(rqhost))
+    {
+	if (debug_flag)
+	    fprintf(stderr, "\t-> Ignored (not a valid source host)\n");
+	
+	return NULL;
+    }
+
     result.transid = xfr->transid;
     result.xfrstat = YPXFR_REFUSED;
     
@@ -697,14 +727,19 @@ void *ypproc_clear_2_svc(void *dummy,
     
 
     rqhost = svc_getcaller(rqstp->rq_xprt);
-    if (!is_valid_host(rqhost))
-	return NULL;
 
     if (debug_flag)
     	fprintf(stderr, "ypproc_clear_2_svc() [From: %s:%d]\n",
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
 		
+    if (!is_valid_host(rqhost))
+    {
+	if (debug_flag)
+	    fprintf(stderr, "\t-> Ignored (not a valid source host)\n");
+	
+	return NULL;
+    }
 
     return (void *) &foo;
 }
@@ -754,8 +789,6 @@ ypresp_all *ypproc_all_2_svc(ypreq_nokey *nokey,
 
     
     rqhost = svc_getcaller(rqstp->rq_xprt);
-    if (!is_valid_host(rqhost))
-	return NULL;
 
     if (debug_flag)
     {
@@ -767,6 +800,14 @@ ypresp_all *ypproc_all_2_svc(ypreq_nokey *nokey,
 	fprintf(stderr, "\t\tmap = \"%s\"\n", nokey->map);
     }
 
+    if (!is_valid_host(rqhost))
+    {
+	if (debug_flag)
+	    fprintf(stderr, "\t-> Ignored (not a valid source host)\n");
+	
+	return NULL;
+    }
+
     xdr_ypall_cb.u.encode = NULL;
     xdr_ypall_cb.u.close  = NULL;
     xdr_ypall_cb.data = NULL;
@@ -823,8 +864,6 @@ ypresp_master *ypproc_master_2_svc(ypreq_nokey *nokey,
     
     
     rqhost = svc_getcaller(rqstp->rq_xprt);
-    if (!is_valid_host(rqhost))
-	return NULL;
 
     if (debug_flag)
     {
@@ -836,6 +875,14 @@ ypresp_master *ypproc_master_2_svc(ypreq_nokey *nokey,
 	fprintf(stderr, "\t\tmap = \"%s\"\n", nokey->map);
     }
 
+    if (!is_valid_host(rqhost))
+    {
+	if (debug_flag)
+	    fprintf(stderr, "\t-> Ignored (not a valid source host)\n");
+	
+	return NULL;
+    }
+
     if (result.peer)
     {
 	free(result.peer);
@@ -893,8 +940,6 @@ ypresp_order *ypproc_order_2_svc(ypreq_nokey *nokey,
     
     
     rqhost = svc_getcaller(rqstp->rq_xprt);
-    if (!is_valid_host(rqhost))
-	return NULL;
 
     if (debug_flag)
     {
@@ -906,6 +951,14 @@ ypresp_order *ypproc_order_2_svc(ypreq_nokey *nokey,
 	fprintf(stderr, "\t\tmap = \"%s\"\n", nokey->map);
     }
 
+    if (!is_valid_host(rqhost))
+    {
+	if (debug_flag)
+	    fprintf(stderr, "\t-> Ignored (not a valid source host)\n");
+	
+	return NULL;
+    }
+
     result.ordernum = 0;
 
     if (nokey->domain[0] == '\0')
@@ -992,8 +1045,6 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name,
     
 
     rqhost = svc_getcaller(rqstp->rq_xprt);
-    if (!is_valid_host(rqhost))
-	return NULL;
 
     if (debug_flag)
     {
@@ -1004,6 +1055,14 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name,
 	fprintf(stderr, "\t\tdomain = \"%s\"\n", *name);
     }
 
+    if (!is_valid_host(rqhost))
+    {
+	if (debug_flag)
+	    fprintf(stderr, "\t-> Ignored (not a valid source host)\n");
+	
+	return NULL;
+    }
+
     if (result.maps)
 	free_maplist(result.maps);
     
@@ -1017,11 +1076,11 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name,
     {
 	DIR *dp;
 
-	dp = opendir(path_ypdb);
+	dp = opendir(".");
 	if (dp == NULL)
 	{
 	    if (debug_flag)
-		perror("opendir");
+		perror("ypserv: opendir: .");
 	    
 	    result.stat = YP_BADDB;
 	}
